__title__ = 'fobi.contrib.plugins.form_handlers.mail.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'MULTI_EMAIL_FIELD_VALUE_SPLITTER',
)

from .conf import get_setting

MULTI_EMAIL_FIELD_VALUE_SPLITTER = get_setting(
    'MULTI_EMAIL_FIELD_VALUE_SPLITTER'
    )



from __future__ import absolute_import

__title__ = 'fobi.contrib.plugins.form_elements.fields.time.apps'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('Config',)

try:
    from django.apps import AppConfig

    class Config(AppConfig):
        name = 'fobi.contrib.plugins.form_elements.fields.time'
        label = 'fobi_contrib_plugins_form_elements_fields_time'

except ImportError:
    pass



__title__ = 'fobi.urls.view'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('urlpatterns',)

from django.conf.urls import url #patterns, include
from django.utils.translation import ugettext_lazy as _
from fobi.views import (
    form_entry_submitted, view_form_entry, form_entry_submitted
    )

#urlpatterns = patterns('fobi.views',
urlpatterns = [
    # Form submitted success page
    url(_(r'^view/submitted/$'),
        view=form_entry_submitted,
        name='fobi.form_entry_submitted'),

    # View form entry
    url(_(r'^view/(?P<form_entry_slug>[\w_\-]+)/$'),
        view_form_entry,
        name='fobi.view_form_entry'),

    # Form submitted success page
    url(_(r'^view/(?P<form_entry_slug>[\w_\-]+)/submitted/$'),
        view=form_entry_submitted,
        name='fobi.form_entry_submitted'),
#)
]



"""
- `RESTRICT_PLUGIN_ACCESS` (bool): If set to True, (Django) permission system
  for fobi plugins is enabled.
- `FORM_ELEMENT_PLUGINS_MODULE_NAME` (str): Name of the module to placed in 
  the (external) apps in which the fobi form element plugin code should be
  implemented and registered.
- `FORM_HANDLER_PLUGINS_MODULE_NAME` (str): Name of the module to placed in 
  the (external) apps in which the fobi form handler plugin code should be
  implemented and registered.
- `FORM_CALLBACKS_MODULE_NAME` (str): Name of the module to placed in the 
  (external) apps in which the fobi form callback code should be implemented
  and registered.
- `FORM_HANDLER_PLUGINS_EXECUTION_ORDER` (tuple): Order in which the form 
  handler plugins are to be executed.
- `DEBUG`
"""
__title__ = 'fobi.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = (
    'RESTRICT_PLUGIN_ACCESS', 'FORM_ELEMENT_PLUGINS_MODULE_NAME',
    'FORM_HANDLER_PLUGINS_MODULE_NAME', 'FORM_IMPORTER_PLUGINS_MODULE_NAME',
    'FORM_CALLBACKS_MODULE_NAME', 'THEMES_MODULE_NAME', 'DEFAULT_THEME',
    'DISPLAY_AUTH_LINK', 'DEBUG', 'GET_PARAM_INITIAL_DATA',

    'CUSTOM_THEME_DATA', 'THEME_FOOTER_TEXT',

    'DEFAULT_MAX_LENGTH', 'FORM_HANDLER_PLUGINS_EXECUTION_ORDER',
    'FAIL_ON_MISSING_FORM_ELEMENT_PLUGINS',
    'FAIL_ON_MISSING_FORM_HANDLER_PLUGINS',
    'FAIL_ON_ERRORS_IN_FORM_ELEMENT_PLUGINS',
    'FAIL_ON_ERRORS_IN_FORM_HANDLER_PLUGINS',

    'WAIT_BETWEEN_TEST_STEPS', 'WAIT_AT_TEST_END',
)

from fobi.conf import get_setting
from fobi.exceptions import NoDefaultThemeSet

# **************************************************************
# **************************************************************
# *************************** Core *****************************
# **************************************************************
# **************************************************************
RESTRICT_PLUGIN_ACCESS = get_setting('RESTRICT_PLUGIN_ACCESS')

FORM_ELEMENT_PLUGINS_MODULE_NAME = \
    get_setting('FORM_ELEMENT_PLUGINS_MODULE_NAME')

FORM_HANDLER_PLUGINS_MODULE_NAME = \
    get_setting('FORM_HANDLER_PLUGINS_MODULE_NAME')

FORM_IMPORTER_PLUGINS_MODULE_NAME = \
    get_setting('FORM_IMPORTER_PLUGINS_MODULE_NAME')

FORM_CALLBACKS_MODULE_NAME = get_setting('FORM_CALLBACKS_MODULE_NAME')

THEMES_MODULE_NAME = get_setting('THEMES_MODULE_NAME')

DEFAULT_THEME = get_setting('DEFAULT_THEME')

DISPLAY_AUTH_LINK = get_setting('DISPLAY_AUTH_LINK')

GET_PARAM_INITIAL_DATA = get_setting('GET_PARAM_INITIAL_DATA')

if not DEFAULT_THEME:
    raise NoDefaultThemeSet("No default theme set!")

DEBUG = get_setting('DEBUG')

# **************************************************************
# **************************************************************
# ************************ Theme related ***********************
# **************************************************************
# **************************************************************
CUSTOM_THEME_DATA = get_setting('CUSTOM_THEME_DATA')
THEME_FOOTER_TEXT = get_setting('THEME_FOOTER_TEXT')

# **************************************************************
# **************************************************************
# *********************** Plugin related ***********************
# **************************************************************
# **************************************************************
DEFAULT_MAX_LENGTH = get_setting('DEFAULT_MAX_LENGTH')

FORM_HANDLER_PLUGINS_EXECUTION_ORDER = \
    get_setting('FORM_HANDLER_PLUGINS_EXECUTION_ORDER')

FAIL_ON_MISSING_FORM_ELEMENT_PLUGINS = \
    get_setting('FAIL_ON_MISSING_FORM_ELEMENT_PLUGINS')

FAIL_ON_MISSING_FORM_HANDLER_PLUGINS = \
    get_setting('FAIL_ON_MISSING_FORM_HANDLER_PLUGINS')

FAIL_ON_ERRORS_IN_FORM_ELEMENT_PLUGINS = \
    get_setting('FAIL_ON_ERRORS_IN_FORM_ELEMENT_PLUGINS')

FAIL_ON_ERRORS_IN_FORM_HANDLER_PLUGINS = \
    get_setting('FAIL_ON_ERRORS_IN_FORM_HANDLER_PLUGINS')

# **************************************************************
# **************************************************************
# ************************ Tests related ***********************
# **************************************************************
# **************************************************************
WAIT_BETWEEN_TEST_STEPS = get_setting('WAIT_BETWEEN_TEST_STEPS')
WAIT_AT_TEST_END = get_setting('WAIT_AT_TEST_END')



__title__ = 'fobi.templatetags.future_compat'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('firstof',)

try:
    # We're using the Django 1.6 admin templates, that make use of new
    # things. One of the new additions (changed) was the ``firstof``
    # template tag. If we can't import it, we simply define it outselves
    from django.template.deafulttags import firstof
except ImportError:
    import warnings

    from django.template.base import (
        Node, TemplateSyntaxError
        )
    from django.template import Library
    from django.utils.timezone import template_localtime
    from django.utils.formats import localize
    from django.utils.encoding import force_text
    from django.utils.html import escape
    from django.utils.safestring import mark_safe, EscapeData, SafeData

    register = Library()

    def render_value_in_context(value, context):
        """
        Converts any value to a string to become part of a rendered template. This
        means escaping, if required, and conversion to a unicode object. If value
        is a string, it is expected to have already been translated.
        """
        value = template_localtime(value, use_tz=context.use_tz)
        value = localize(value, use_l10n=context.use_l10n)
        value = force_text(value)
        if ((context.autoescape and not isinstance(value, SafeData)) or
                isinstance(value, EscapeData)):
            return escape(value)
        else:
            return value

    class FirstOfNode(Node):
        def __init__(self, variables, escape=False):
            self.vars = variables
            self.escape = escape        # only while the "future" version exists

        def render(self, context):
            for var in self.vars:
                value = var.resolve(context, True)
                if value:
                    if not self.escape:
                        value = mark_safe(value)
                    return render_value_in_context(value, context)
            return ''


    @register.tag
    def firstof(parser, token, escape=False):
        """
        Outputs the first variable passed that is not False, without escaping.

        Outputs nothing if all the passed variables are False.

        Sample usage::

            {% firstof var1 var2 var3 %}

        This is equivalent to::

            {% if var1 %}
                {{ var1|safe }}
            {% elif var2 %}
                {{ var2|safe }}
            {% elif var3 %}
                {{ var3|safe }}
            {% endif %}

        but obviously much cleaner!

        You can also use a literal string as a fallback value in case all
        passed variables are False::

            {% firstof var1 var2 var3 "fallback value" %}

        If you want to escape the output, use a filter tag::

            {% filter force_escape %}
                {% firstof var1 var2 var3 "fallback value" %}
            {% endfilter %}

        """
        if not escape:
            warnings.warn(
                "'The `firstof` template tag is changing to escape its arguments; "
                "the non-autoescaping version is deprecated. Load it "
                "from the `future` tag library to start using the new behavior.",
                DeprecationWarning, stacklevel=2)

        bits = token.split_contents()[1:]
        if len(bits) < 1:
            raise TemplateSyntaxError("'firstof' statement requires at least one argument")
        return FirstOfNode([parser.compile_filter(bit) for bit in bits], escape=escape)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models, migrations
import autoslug.fields
from django.conf import settings
from nine.versions import DJANGO_GTE_1_8


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    if DJANGO_GTE_1_8:
        dependencies += [('auth', '0006_require_contenttypes_0002'),]

    operations = [
        migrations.CreateModel(
            name='FormElement',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('plugin_uid', models.CharField(verbose_name='Plugin UID', unique=True, max_length=255, editable=False, choices=[(b'select_model_object', b'Select model object'), (b'text', b'Text'), (b'float', b'Float'), (b'datetime', b'DateTime'), (b'boolean', b'Boolean'), (b'radio', b'Radio'), (b'file', b'File'), (b'select', b'Select'), (b'regex', b'Regex'), (b'content_image', b'Content image'), (b'null_boolean', b'Null boolean'), (b'input', b'Input'), (b'hidden', b'Hidden'), (b'email', b'Email'), (b'select_multiple_model_objects', b'Select multiple model objects'), (b'select_multiple', b'Select multiple'), (b'checkbox_select_multiple', b'Checkbox select multiple'), (b'content_text', b'Content text'), (b'date', b'Date'), (b'integer', b'Integer'), (b'password', b'Password'), (b'ip_address', b'IP address'), (b'slug', b'Slug'), (b'dummy', b'Dummy'), (b'textarea', b'Textarea'), (b'url', b'URL'), (b'decimal', b'Decimal'), (b'content_video', b'Content video'), (b'time', b'Time'), (b'date_drop_down', b'Date drop down'), (b'honeypot', b'Honeypot')])),
                ('groups', models.ManyToManyField(to='auth.Group', verbose_name='Group', blank=True)),
                ('users', models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name='User', blank=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Form element plugin',
                'verbose_name_plural': 'Form element plugins',
            },
        ),
        migrations.CreateModel(
            name='FormElementEntry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('plugin_data', models.TextField(null=True, verbose_name='Plugin data', blank=True)),
                ('plugin_uid', models.CharField(max_length=255, verbose_name='Plugin name', choices=[(b'select_model_object', b'Select model object'), (b'text', b'Text'), (b'float', b'Float'), (b'datetime', b'DateTime'), (b'boolean', b'Boolean'), (b'radio', b'Radio'), (b'file', b'File'), (b'select', b'Select'), (b'regex', b'Regex'), (b'content_image', b'Content image'), (b'null_boolean', b'Null boolean'), (b'input', b'Input'), (b'hidden', b'Hidden'), (b'email', b'Email'), (b'select_multiple_model_objects', b'Select multiple model objects'), (b'select_multiple', b'Select multiple'), (b'checkbox_select_multiple', b'Checkbox select multiple'), (b'content_text', b'Content text'), (b'date', b'Date'), (b'integer', b'Integer'), (b'password', b'Password'), (b'ip_address', b'IP address'), (b'slug', b'Slug'), (b'dummy', b'Dummy'), (b'textarea', b'Textarea'), (b'url', b'URL'), (b'decimal', b'Decimal'), (b'content_video', b'Content video'), (b'time', b'Time'), (b'date_drop_down', b'Date drop down'), (b'honeypot', b'Honeypot')])),
                ('position', models.PositiveIntegerField(null=True, verbose_name='Position', blank=True)),
            ],
            options={
                'ordering': ['position'],
                'abstract': False,
                'verbose_name': 'Form element entry',
                'verbose_name_plural': 'Form element entries',
            },
        ),
        migrations.CreateModel(
            name='FormEntry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('slug', autoslug.fields.AutoSlugField(verbose_name='Slug', unique=True, editable=False)),
                ('is_public', models.BooleanField(default=False, help_text='Makes your form visible to the public.', verbose_name='Public?')),
                ('is_cloneable', models.BooleanField(default=False, help_text='Makes your form cloneable by other users.', verbose_name='Cloneable?')),
                ('position', models.PositiveIntegerField(null=True, verbose_name='Position', blank=True)),
                ('success_page_title', models.CharField(help_text='Custom message title to display after valid form is submitted', max_length=255, null=True, verbose_name='Success page title', blank=True)),
                ('success_page_message', models.TextField(help_text='Custom message text to display after valid form is submitted', null=True, verbose_name='Success page body', blank=True)),
                ('action', models.CharField(help_text="Custom form action; don't fill this field, unless really necessary.", max_length=255, null=True, verbose_name='Action', blank=True)),
            ],
            options={
                'verbose_name': 'Form entry',
                'verbose_name_plural': 'Form entries',
            },
        ),
        migrations.CreateModel(
            name='FormFieldsetEntry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('is_repeatable', models.BooleanField(default=False, help_text='Makes your form fieldset repeatable.', verbose_name='Is repeatable?')),
                ('form_entry', models.ForeignKey(verbose_name='Form', blank=True, to='fobi.FormEntry', null=True)),
            ],
            options={
                'verbose_name': 'Form fieldset entry',
                'verbose_name_plural': 'Form fieldset entries',
            },
        ),
        migrations.CreateModel(
            name='FormHandler',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('plugin_uid', models.CharField(verbose_name='Plugin UID', unique=True, max_length=255, editable=False, choices=[(b'db_store', b'DB store'), (b'mail', b'Mail'), (b'http_repost', b'HTTP Repost')])),
                ('groups', models.ManyToManyField(to='auth.Group', verbose_name='Group', blank=True)),
                ('users', models.ManyToManyField(to=settings.AUTH_USER_MODEL, verbose_name='User', blank=True)),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Form handler plugin',
                'verbose_name_plural': 'Form handler plugins',
            },
        ),
        migrations.CreateModel(
            name='FormHandlerEntry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('plugin_data', models.TextField(null=True, verbose_name='Plugin data', blank=True)),
                ('plugin_uid', models.CharField(max_length=255, verbose_name='Plugin name', choices=[(b'db_store', b'DB store'), (b'mail', b'Mail'), (b'http_repost', b'HTTP Repost')])),
                ('form_entry', models.ForeignKey(verbose_name='Form', to='fobi.FormEntry')),
            ],
            options={
                'abstract': False,
                'verbose_name': 'Form handler entry',
                'verbose_name_plural': 'Form handler entries',
            },
        ),
        migrations.CreateModel(
            name='FormWizardEntry',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('slug', autoslug.fields.AutoSlugField(verbose_name='Slug', unique=True, editable=False)),
                ('is_public', models.BooleanField(default=False, help_text='Makes your form wizard visible to the public.', verbose_name='Is public?')),
                ('is_cloneable', models.BooleanField(default=False, help_text='Makes your form wizard cloneable by other users.', verbose_name='Is cloneable?')),
                ('user', models.ForeignKey(verbose_name='User', to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Form wizard entry',
                'verbose_name_plural': 'Form wizard entries',
            },
        ),
        migrations.AddField(
            model_name='formentry',
            name='form_wizard_entry',
            field=models.ForeignKey(verbose_name='Form wizard', blank=True, to='fobi.FormWizardEntry', null=True),
        ),
        migrations.AddField(
            model_name='formentry',
            name='user',
            field=models.ForeignKey(verbose_name='User', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='formelemententry',
            name='form_entry',
            field=models.ForeignKey(verbose_name='Form', to='fobi.FormEntry'),
        ),
        migrations.AddField(
            model_name='formelemententry',
            name='form_fieldset_entry',
            field=models.ForeignKey(verbose_name='Form fieldset', blank=True, to='fobi.FormFieldsetEntry', null=True),
        ),
        migrations.AlterUniqueTogether(
            name='formwizardentry',
            unique_together=set([('user', 'slug'), ('user', 'name')]),
        ),
        migrations.AlterUniqueTogether(
            name='formfieldsetentry',
            unique_together=set([('form_entry', 'name')]),
        ),
        migrations.AlterUniqueTogether(
            name='formentry',
            unique_together=set([('user', 'slug'), ('user', 'name')]),
        ),
    ]



"""
- ``HONEYPOT_VALUE`` (string)
"""

__title__ = 'fobi.contrib.plugins.form_elements.security.honeypot.settings'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('HONEYPOT_VALUE',)

from .conf import get_setting

HONEYPOT_VALUE = get_setting('HONEYPOT_VALUE')



__title__ = 'fobi.tests.test_core'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('FobiCoreTest',)

import unittest

from django.test import TestCase, RequestFactory

from fobi.base import (
    get_registered_form_element_plugins, get_registered_form_handler_plugins,
    get_registered_themes, get_registered_form_callbacks
)
from fobi.models import FormEntry
from fobi.forms import FormEntryForm
from fobi.tests.constants import TEST_FORM_NAME, TEST_FORM_SLUG
from fobi.tests.base import print_info
from fobi.tests.helpers import setup_fobi, get_or_create_admin_user

class FobiCoreTest(TestCase):
    """
    Tests of django-fobi core functionality.
    """
    def setUp(self):
        setup_fobi(fobi_sync_plugins=True)

    @print_info
    def test_01_get_registered_form_element_plugins(self):
        """
        Test registered form element plugins
        (`get_registered_form_element_plugins`).
        """
        res = get_registered_form_element_plugins()
        self.assertTrue(len(res) > 0)
        return res

    @print_info
    def test_02_get_registered_form_handler_plugins(self):
        """
        Test registered form handlers (`get_registered_form_handler_plugins`).
        """
        res = get_registered_form_handler_plugins()
        self.assertTrue(len(res) > 0)
        return res

    @print_info
    def test_03_get_registered_form_callbacks(self):
        """
        Test registered form callbacks (`get_registered_form_callbacks`).
        """
        res = get_registered_form_callbacks()
        self.assertTrue(len(res) > 0)
        return res

    @print_info
    def test_04_get_registered_themes(self):
        """
        Test registered themes (`get_registered_themes`).
        """
        res = get_registered_themes()
        self.assertTrue(len(res) > 0)
        return res

    def _test_form_action_url(self, form_entry, action_url):
        request_factory = RequestFactory()
        request = request_factory.post(
            '/en/fobi/forms/edit/27/',
            data = {
                'name': "John Doe",
                'is_public': False,
                'success_page_title': '',
                'success_page_message': '',
                'action': action_url,
            }
            )
        request.META['SERVER_NAME'] = 'localhost'
        form = FormEntryForm(request.POST, request=request, instance=form_entry)

        saved = False
        try:

            if form.is_valid():
                form.save()
                saved = True
        except Exception as err:
            pass

        return saved

    def _create_form_entry(self):
        user = get_or_create_admin_user()
        self.assertTrue(user is not None)

        form_entry = FormEntry(
            name = TEST_FORM_NAME,
            slug = TEST_FORM_SLUG,
            user = user
            )
        form_entry.save()
        return form_entry

    @print_info
    def test_05_action_url(self):
        """
        Test `action` field of the URL.
        """
        form_entry = self._create_form_entry()

        # Local URL, OK test
        saved = self._test_form_action_url(
            form_entry, '/en/fobi/forms/edit/27/'
            )
        self.assertTrue(saved)

        # Local URL, fail test
        saved = self._test_form_action_url(
            form_entry, '/en/idontexist/'
            )
        self.assertTrue(not saved)

        # External URL, OK test
        saved = self._test_form_action_url(
            form_entry, 'http://delusionalinsanity.com/portfolio/'
            )
        self.assertTrue(saved)

        # External URL, fail test
        saved = self._test_form_action_url(
            form_entry, 'http://delusionalinsanity.com2/portfolio/'
            )
        self.assertTrue(not saved)

        # External URL, fail test
        saved = self._test_form_action_url(
            form_entry, 'http://delusionalinsanity2.com/portfolio/'
            )
        self.assertTrue(not saved)


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

from django.core.management.base import BaseCommand

from fobi.models import FormElementEntry

class Command(BaseCommand):
    def handle(self, *args, **options):
        """
        Database related changes necessary to upgrade fobi==0.3.* to
        fobi==0.4. The full list of changes is listed below:

        - Change the "birthday" occurances to "date_drop_down".
        """
        n_updated = FormElementEntry._default_manager \
                                    .filter(plugin_uid='birthday') \
                                    .only('id', 'plugin_uid') \
                                    .update(plugin_uid='date_drop_down')

        print("{0} form element entries updated!".format(n_updated))



__title__ = 'fobi.dynamic'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = '2014-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('assemble_form_class',)

#import logging

from six import with_metaclass

from django.forms.forms import BaseForm#, get_declared_fields
from django.forms.widgets import media_property

from nine.versions import DJANGO_GTE_1_7

if DJANGO_GTE_1_7:
    from collections import OrderedDict
else:
    from django.utils.datastructures import SortedDict as OrderedDict

#logger = logging.getLogger(__file__)

# ****************************************************************************
# ****************************************************************************
# **************************** Form generator ********************************
# ****************************************************************************
# ****************************************************************************

def assemble_form_class(form_entry, base_class=BaseForm, request=None,
                        origin=None, origin_kwargs_update_func=None,
                        origin_return_func=None, form_element_entries=None):
    """
    Assembles a form class by given entry.

    :param form_entry:
    :param base_class:
    :param django.http.HttpRequest request:
    :param string origin:
    :param callable origin_kwargs_update_func:
    :param callable origin_return_func:
    :param iterable form_element_entries: If given, used instead of
        ``form_entry.formelemententry_set.all`` (no additional database hit).
    """
    if form_element_entries is None:
        form_element_entries = form_entry.formelemententry_set.all()

    class DeclarativeFieldsMetaclass(type):
        """
        Copied from ``django.forms.forms.DeclarativeFieldsMetaclass``.

        Metaclass that converts Field attributes to a dictionary called
        `base_fields`, taking into account parent class 'base_fields' as well.
        """
        def __new__(cls, name, bases, attrs):
            base_fields = []

            for creation_counter, form_element_entry \
                    in enumerate(form_element_entries):
                plugin = form_element_entry.get_plugin(request=request)

                # We simply make sure the plugin exists. We don't handle
                # exceptions relate to the non-existent plugins here. They
                # are instead handled in registry.
                if plugin:
                    plugin_form_field_instances = \
                        plugin._get_form_field_instances(
                            form_element_entry = form_element_entry,
                            origin = origin,
                            kwargs_update_func = origin_kwargs_update_func,
                            return_func = origin_return_func,
                            extra = {'counter': creation_counter},
                            request = request
                        )
                    for form_field_name, form_field_instance \
                            in plugin_form_field_instances:
                        base_fields.append(
                                (form_field_name, form_field_instance)
                        )

            attrs['base_fields'] = OrderedDict(base_fields)
            new_class = super(DeclarativeFieldsMetaclass, cls).__new__(
                cls, name, bases, attrs
                )

            if 'media' not in attrs:
                new_class.media = media_property(new_class)

            return new_class

    class DynamicForm(with_metaclass(DeclarativeFieldsMetaclass, base_class)):
        """
        Dynamically created form element plugin class.
        """

    return DynamicForm



__title__ = 'fobi.context_processors'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('theme', 'dynamic_values', 'form_importers',)

import datetime

from fobi.base import get_theme
from fobi.helpers import StrippedRequest
from fobi.form_importers import get_form_impoter_plugin_urls

def theme(request):
    """
    Gets active theme.

    :param django.http.HttpRequest request:
    :return fobi.base.BaseTheme: Instance of ``fobi.base.BaseTheme``.
    """
    return {'fobi_theme': get_theme(request, as_instance=True)}

def dynamic_values(request):
    """
    Dynamic values exposed to public forms.
    """
    return {
        'fobi_dynamic_values': {
            'request': StrippedRequest(request),
            'now': datetime.datetime.now(),
            'today': datetime.date.today(),
        }
    }

def form_importers(request):
    """
    Form importers.
    """
    return {
        'form_importers': get_form_impoter_plugin_urls(),
    }



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'FormElement'
        db.create_table(u'fobi_formelement', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('plugin_uid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
        ))
        db.send_create_signal(u'fobi', ['FormElement'])

        # Adding M2M table for field users on 'FormElement'
        m2m_table_name = db.shorten_name(u'fobi_formelement_users')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('formelement', models.ForeignKey(orm[u'fobi.formelement'], null=False)),
            ('user', models.ForeignKey(orm[u'auth.user'], null=False))
        ))
        db.create_unique(m2m_table_name, ['formelement_id', 'user_id'])

        # Adding M2M table for field groups on 'FormElement'
        m2m_table_name = db.shorten_name(u'fobi_formelement_groups')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('formelement', models.ForeignKey(orm[u'fobi.formelement'], null=False)),
            ('group', models.ForeignKey(orm[u'auth.group'], null=False))
        ))
        db.create_unique(m2m_table_name, ['formelement_id', 'group_id'])

        # Adding model 'FormHandler'
        db.create_table(u'fobi_formhandler', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('plugin_uid', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255)),
        ))
        db.send_create_signal(u'fobi', ['FormHandler'])

        # Adding M2M table for field users on 'FormHandler'
        m2m_table_name = db.shorten_name(u'fobi_formhandler_users')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('formhandler', models.ForeignKey(orm[u'fobi.formhandler'], null=False)),
            ('user', models.ForeignKey(orm[u'auth.user'], null=False))
        ))
        db.create_unique(m2m_table_name, ['formhandler_id', 'user_id'])

        # Adding M2M table for field groups on 'FormHandler'
        m2m_table_name = db.shorten_name(u'fobi_formhandler_groups')
        db.create_table(m2m_table_name, (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('formhandler', models.ForeignKey(orm[u'fobi.formhandler'], null=False)),
            ('group', models.ForeignKey(orm[u'auth.group'], null=False))
        ))
        db.create_unique(m2m_table_name, ['formhandler_id', 'group_id'])

        # Adding model 'FormWizardEntry'
        db.create_table(u'fobi_formwizardentry', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('slug', self.gf('autoslug.fields.AutoSlugField')(unique=True, max_length=50, populate_from='name', unique_with=())),
            ('is_public', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_cloneable', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'fobi', ['FormWizardEntry'])

        # Adding unique constraint on 'FormWizardEntry', fields ['user', 'slug']
        db.create_unique(u'fobi_formwizardentry', ['user_id', 'slug'])

        # Adding unique constraint on 'FormWizardEntry', fields ['user', 'name']
        db.create_unique(u'fobi_formwizardentry', ['user_id', 'name'])

        # Adding model 'FormEntry'
        db.create_table(u'fobi_formentry', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('form_wizard_entry', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['fobi.FormWizardEntry'], null=True, blank=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('slug', self.gf('autoslug.fields.AutoSlugField')(unique=True, max_length=50, populate_from='name', unique_with=())),
            ('is_public', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('is_cloneable', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('position', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
            ('success_page_title', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
            ('success_page_message', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('action', self.gf('django.db.models.fields.CharField')(max_length=255, null=True, blank=True)),
        ))
        db.send_create_signal(u'fobi', ['FormEntry'])

        # Adding unique constraint on 'FormEntry', fields ['user', 'slug']
        db.create_unique(u'fobi_formentry', ['user_id', 'slug'])

        # Adding unique constraint on 'FormEntry', fields ['user', 'name']
        db.create_unique(u'fobi_formentry', ['user_id', 'name'])

        # Adding model 'FormFieldsetEntry'
        db.create_table(u'fobi_formfieldsetentry', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('form_entry', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['fobi.FormEntry'], null=True, blank=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('is_repeatable', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal(u'fobi', ['FormFieldsetEntry'])

        # Adding unique constraint on 'FormFieldsetEntry', fields ['form_entry', 'name']
        db.create_unique(u'fobi_formfieldsetentry', ['form_entry_id', 'name'])

        # Adding model 'FormElementEntry'
        db.create_table(u'fobi_formelemententry', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('form_entry', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['fobi.FormEntry'])),
            ('plugin_data', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('plugin_uid', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('form_fieldset_entry', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['fobi.FormFieldsetEntry'], null=True, blank=True)),
            ('position', self.gf('django.db.models.fields.PositiveIntegerField')(null=True, blank=True)),
        ))
        db.send_create_signal(u'fobi', ['FormElementEntry'])

        # Adding model 'FormHandlerEntry'
        db.create_table(u'fobi_formhandlerentry', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('form_entry', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['fobi.FormEntry'])),
            ('plugin_data', self.gf('django.db.models.fields.TextField')(null=True, blank=True)),
            ('plugin_uid', self.gf('django.db.models.fields.CharField')(max_length=255)),
        ))
        db.send_create_signal(u'fobi', ['FormHandlerEntry'])


    def backwards(self, orm):
        # Removing unique constraint on 'FormFieldsetEntry', fields ['form_entry', 'name']
        db.delete_unique(u'fobi_formfieldsetentry', ['form_entry_id', 'name'])

        # Removing unique constraint on 'FormEntry', fields ['user', 'name']
        db.delete_unique(u'fobi_formentry', ['user_id', 'name'])

        # Removing unique constraint on 'FormEntry', fields ['user', 'slug']
        db.delete_unique(u'fobi_formentry', ['user_id', 'slug'])

        # Removing unique constraint on 'FormWizardEntry', fields ['user', 'name']
        db.delete_unique(u'fobi_formwizardentry', ['user_id', 'name'])

        # Removing unique constraint on 'FormWizardEntry', fields ['user', 'slug']
        db.delete_unique(u'fobi_formwizardentry', ['user_id', 'slug'])

        # Deleting model 'FormElement'
        db.delete_table(u'fobi_formelement')

        # Removing M2M table for field users on 'FormElement'
        db.delete_table(db.shorten_name(u'fobi_formelement_users'))

        # Removing M2M table for field groups on 'FormElement'
        db.delete_table(db.shorten_name(u'fobi_formelement_groups'))

        # Deleting model 'FormHandler'
        db.delete_table(u'fobi_formhandler')

        # Removing M2M table for field users on 'FormHandler'
        db.delete_table(db.shorten_name(u'fobi_formhandler_users'))

        # Removing M2M table for field groups on 'FormHandler'
        db.delete_table(db.shorten_name(u'fobi_formhandler_groups'))

        # Deleting model 'FormWizardEntry'
        db.delete_table(u'fobi_formwizardentry')

        # Deleting model 'FormEntry'
        db.delete_table(u'fobi_formentry')

        # Deleting model 'FormFieldsetEntry'
        db.delete_table(u'fobi_formfieldsetentry')

        # Deleting model 'FormElementEntry'
        db.delete_table(u'fobi_formelemententry')

        # Deleting model 'FormHandlerEntry'
        db.delete_table(u'fobi_formhandlerentry')


    models = {
        u'auth.group': {
            'Meta': {'object_name': 'Group'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'auth.permission': {
            'Meta': {'ordering': "(u'content_type__app_label', u'content_type__model', u'codename')", 'unique_together': "((u'content_type', u'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        u'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Group']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "u'user_set'", 'blank': 'True', 'to': u"orm['auth.Permission']"}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'fobi.formelement': {
            'Meta': {'object_name': 'FormElement'},
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'plugin_uid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.User']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'fobi.formelemententry': {
            'Meta': {'ordering': "['position']", 'object_name': 'FormElementEntry'},
            'form_entry': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['fobi.FormEntry']"}),
            'form_fieldset_entry': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['fobi.FormFieldsetEntry']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'plugin_data': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'plugin_uid': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'})
        },
        u'fobi.formentry': {
            'Meta': {'unique_together': "(('user', 'slug'), ('user', 'name'))", 'object_name': 'FormEntry'},
            'action': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'form_wizard_entry': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['fobi.FormWizardEntry']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_cloneable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'position': ('django.db.models.fields.PositiveIntegerField', [], {'null': 'True', 'blank': 'True'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': "'name'", 'unique_with': '()'}),
            'success_page_message': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'success_page_title': ('django.db.models.fields.CharField', [], {'max_length': '255', 'null': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        },
        u'fobi.formfieldsetentry': {
            'Meta': {'unique_together': "(('form_entry', 'name'),)", 'object_name': 'FormFieldsetEntry'},
            'form_entry': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['fobi.FormEntry']", 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_repeatable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'fobi.formhandler': {
            'Meta': {'object_name': 'FormHandler'},
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'plugin_uid': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'users': ('django.db.models.fields.related.ManyToManyField', [], {'to': u"orm['auth.User']", 'symmetrical': 'False', 'blank': 'True'})
        },
        u'fobi.formhandlerentry': {
            'Meta': {'object_name': 'FormHandlerEntry'},
            'form_entry': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['fobi.FormEntry']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'plugin_data': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'plugin_uid': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'fobi.formwizardentry': {
            'Meta': {'unique_together': "(('user', 'slug'), ('user', 'name'))", 'object_name': 'FormWizardEntry'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_cloneable': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_public': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('autoslug.fields.AutoSlugField', [], {'unique': 'True', 'max_length': '50', 'populate_from': "'name'", 'unique_with': '()'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['auth.User']"})
        }
    }

    complete_apps = ['fobi']


__title__ = 'fobi.contrib.plugins.form_handlers.db_store.models'
__author__ = 'Artur Barseghyan <artur.barseghyan@gmail.com>'
__copyright__ = 'Copyright (c) 2014-2015 Artur Barseghyan'
__license__ = 'GPL 2.0/LGPL 2.1'
__all__ = ('SavedFormDataEntry',)

import simplejson as json

from six import string_types

from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.db import models

# ****************************************************************************
# **************** Safe User import for Django > 1.5, < 1.8 ******************
# ****************************************************************************
AUTH_USER_MODEL = settings.AUTH_USER_MODEL

# ****************************************************************************
# ****************************************************************************
# ****************************************************************************

from fobi.helpers import two_dicts_to_string

class SavedFormDataEntry(models.Model):
    """
    Saved form data.
    """
    form_entry = models.ForeignKey('fobi.FormEntry', verbose_name=_("Form"),
                                   null=True, blank=True)
    user = models.ForeignKey(AUTH_USER_MODEL, verbose_name=_("User"),
                             null=True, blank=True)
    form_data_headers = models.TextField(_("Form data headers"), null=True,
                                         blank=True)
    saved_data = models.TextField(_("Plugin data"), null=True, blank=True)
    created = models.DateTimeField(_("Date created"), auto_now_add=True)

    class Meta:
        abstract = False
        verbose_name = _("Saved form data entry")
        verbose_name_plural = _("Saved form data entries")
        db_table = 'db_store_savedformdataentry'

    def __unicode__(self):
        return "Saved form data entry from {0}".format(self.created)

    def formatted_saved_data(self):
        """
        Shows the formatted saved data records.

        :return string:
        """
        try:
            headers = json.loads(self.form_data_headers)
            data = json.loads(self.saved_data)
            for key, value in data.items():
                if isinstance(value, string_types) and \
                   (value.startswith(settings.MEDIA_URL) or \
                    value.startswith('http://') or value.startswith('https://')):

                    data[key] = '<a href="{value}">{value}</a>'.format(value=value)

            return two_dicts_to_string(headers, data)
        except (ValueError, json.decoder.JSONDecodeError) as err:
            return ''
    formatted_saved_data.allow_tags = True
    formatted_saved_data.short_description = _("Saved data")
